#ifndef _COMPAT_ARCH_X86_XEN_MCA_H
#define _COMPAT_ARCH_X86_XEN_MCA_H
#include <xen/compat.h>
struct compat_mcinfo_common {
    uint16_t type;
    uint16_t size;
};
typedef struct mcinfo_common __attribute__((__aligned__(__alignof( struct compat_mcinfo_common)))) compat_mcinfo_common_t;
struct compat_mcinfo_global {
    compat_mcinfo_common_t common;

    uint16_t mc_domid;
    uint16_t mc_vcpuid;
    uint32_t mc_socketid;
    uint16_t mc_coreid;
    uint16_t mc_core_threadid;
    uint32_t mc_apicid;
    uint32_t mc_flags;
    uint64_compat_t mc_gstatus;
};

struct compat_mcinfo_bank {
    compat_mcinfo_common_t common;

    uint16_t mc_bank;
    uint16_t mc_domid;

    uint64_compat_t mc_status;
    uint64_compat_t mc_addr;

    uint64_compat_t mc_misc;
    uint64_compat_t mc_ctrl2;
    uint64_compat_t mc_tsc;
};

struct compat_mcinfo_msr {
    uint64_compat_t reg;
    uint64_compat_t value;
};
typedef struct mcinfo_msr __attribute__((__aligned__(__alignof( struct compat_mcinfo_msr)))) compat_mcinfo_msr_t;

struct compat_mcinfo_extended {
    compat_mcinfo_common_t common;

    uint32_t mc_msrs;

    compat_mcinfo_msr_t mc_msr[32];
};
struct compat_page_offline_action
{

    uint64_compat_t mfn;
    uint64_compat_t status;
};
typedef struct page_offline_action __attribute__((__aligned__(__alignof( struct compat_page_offline_action)))) compat_page_offline_action_t;

struct compat_cpu_offline_action
{

    uint32_t mc_socketid;
    uint16_t mc_coreid;
    uint16_t mc_core_threadid;
};
typedef struct cpu_offline_action __attribute__((__aligned__(__alignof( struct compat_cpu_offline_action)))) compat_cpu_offline_action_t;

struct compat_mcinfo_recovery
{
    compat_mcinfo_common_t common;
    uint16_t mc_bank;
    uint8_t action_flags;
    uint8_t action_types;
    union {
        compat_page_offline_action_t page_retire;
        compat_cpu_offline_action_t cpu_offline;
        uint8_t pad[16];
    } action_info;
};

struct compat_mc_info {

    uint32_t mi_nentries;
    uint32_t flags;
    uint64_compat_t mi_data[(768 - 1) / 8];
};
typedef struct mc_info __attribute__((__aligned__(__alignof( struct compat_mc_info)))) mc_info_compat_t;
DEFINE_COMPAT_HANDLE(mc_info_compat_t);
struct compat_mcinfo_logical_cpu {
    uint32_t mc_cpunr;
    uint32_t mc_chipid;
    uint16_t mc_coreid;
    uint16_t mc_threadid;
    uint32_t mc_apicid;
    uint32_t mc_clusterid;
    uint32_t mc_ncores;
    uint32_t mc_ncores_active;
    uint32_t mc_nthreads;
    int32_t mc_cpuid_level;
    uint32_t mc_family;
    uint32_t mc_vendor;
    uint32_t mc_model;
    uint32_t mc_step;
    char mc_vendorid[16];
    char mc_brandid[64];
    uint32_t mc_cpu_caps[7];
    uint32_t mc_cache_size;
    uint32_t mc_cache_alignment;
    int32_t mc_nmsrvals;
    compat_mcinfo_msr_t mc_msrvalues[8];
};
typedef struct mcinfo_logical_cpu __attribute__((__aligned__(__alignof( struct compat_mcinfo_logical_cpu)))) compat_mc_logical_cpu_t;
DEFINE_COMPAT_HANDLE(compat_mc_logical_cpu_t);
struct compat_mc_fetch {

    uint32_t flags;

    uint32_t _pad0;
    uint64_compat_t fetch_id;

    COMPAT_HANDLE(mc_info_compat_t) data;
};
typedef struct compat_mc_fetch compat_mc_fetch_t;
DEFINE_COMPAT_HANDLE(compat_mc_fetch_t);

struct compat_mc_notifydomain {

    uint16_t mc_domid;
    uint16_t mc_vcpuid;

    uint32_t flags;

};
typedef struct xen_mc_notifydomain __attribute__((__aligned__(__alignof( struct compat_mc_notifydomain)))) compat_mc_notifydomain_t;
DEFINE_COMPAT_HANDLE(compat_mc_notifydomain_t);

struct compat_mc_physcpuinfo {

    uint32_t ncpus;
    uint32_t _pad0;

    COMPAT_HANDLE(compat_mc_logical_cpu_t) info;
};
typedef struct compat_mc_physcpuinfo compat_mc_physcpuinfo_t;

struct compat_mc_msrinject {

    uint32_t mcinj_cpunr;
    uint32_t mcinj_flags;
    uint32_t mcinj_count;
    domid_compat_t mcinj_domid;

    uint16_t _pad0;
    compat_mcinfo_msr_t mcinj_msr[8];
};
typedef struct xen_mc_msrinject __attribute__((__aligned__(__alignof( struct compat_mc_msrinject)))) compat_mc_msrinject_t;

struct compat_mc_mceinject {
    unsigned int mceinj_cpunr;
};
typedef struct xen_mc_mceinject __attribute__((__aligned__(__alignof( struct compat_mc_mceinject)))) compat_mc_mceinject_t;
struct compat_mc_inject_v2 {
    uint32_t flags;
    compat_ctl_bitmap_t cpumap;
};
typedef struct xen_mc_inject_v2 __attribute__((__aligned__(__alignof( struct compat_mc_inject_v2)))) compat_mc_inject_v2_t;

struct compat_mc {
    uint32_t cmd;
    uint32_t interface_version;
    union {
        compat_mc_fetch_t mc_fetch;
        compat_mc_notifydomain_t mc_notifydomain;
        compat_mc_physcpuinfo_t mc_physcpuinfo;
        compat_mc_msrinject_t mc_msrinject;
        compat_mc_mceinject_t mc_mceinject;

        compat_mc_inject_v2_t mc_inject_v2;

    } u;
};
typedef struct xen_mc __attribute__((__aligned__(__alignof( struct compat_mc)))) compat_mc_t;
DEFINE_COMPAT_HANDLE(compat_mc_t);
#endif /* _COMPAT_ARCH_X86_XEN_MCA_H */
